#!/bin/bash
# vpn accounts management - shell script for openvpn
# usage : vpnmgmt(.sh) {add/remove/public} {username,mailaddress}
# requirements : at(d) daemon - installed & started

# - variables
# mysql database details
db_user="user"
db_pass="pass"
db_name="name"
# openvpn
run_path="/root/work/vpnmgmt"
conf="/etc/openvpn"
md5sumbin=`which md5sum`
mailxbin=`which mail`
urltime=24 #this is in HOURS
mailcheck="webops@example.org"
weburl="http://www.example.org/vpn"

# - functions
function addvpnuser
    {
        echo "- adding user $2"
        #cd /usr/share/doc/openvpn/examples/easy-rsa/2.0
        cd /etc/openvpn/easy-rsa/2.0
        cp vars $2
        sed -i -e "s/KEY_CN=vpn.example.org/KEY_CN=$2/g" $2
        sed -i -e "s/KEY_NAME=webops/KEY_NAME=$2/g" $2
        sed -i -e "s/KEY_OU=webops/KEY_OU=$2/g" $2
        source ./$2
        ./build-key $2
        cp -vu keys/${2}.* /etc/openvpn/
        password=`pwgen -s 16 | head -1 | awk '{print $1}'`
        mysql -Bsu $db_user -p$db_pass $db_name -e "insert into user set user_id=\"$2\", user_pass=\"$password\"";
        if [ -f /etc/openvpn/${2}.key ]; then
            echo "VPN user created"
            echo "user: $2"
            echo "pass: $password"
            #`ls -al $conf | grep $2` > $run_path/acct/add/$2
            echo `date` >> $run_path/acct/accounting.txt
            echo "Add :" >> $run_path/acct/accounting.txt
            echo $2 >> $run_path/acct/accounting.txt
            echo "-----------" >> $run_path/acct/accounting.txt
            cd /etc/openvpn/easy-rsa/2.0/keys/
            #mv `ls -1t /etc/openvpn/easy-rsa/2.0/keys/*.pem | head -1` $2.pem
	    #echo "added $2" | $mailxbin -s "vpn add $2 - $weburl" ${mailcheck}
        else
            echo "$2 - adding vpn user encountered problems; please retry"
        fi
    }

function rmvpnuser
    {
        echo "- removing user $2"
        cd /etc/openvpn/easy-rsa/2.0
        source ./vars
        /etc/openvpn/easy-rsa/2.0/revoke-full $2
        #rm -v $conf/${2}.*
        #rm -v $conf/easy-rsa/2.0/keys/${2}.*
        #rm -v /etc/openvpn/easy-rsa/2.0/keys/$2.pem
        #rm -v $conf/acct/add/$2
        #`date` > $run_path/acct/rm/$2
        mysql -Bsu $db_user -p$db_pass $db_name -e "delete from user where user_id=\"$2\"";
        echo `date` >> $run_path/acct/accounting.txt
        echo "Remove :" >> $run_path/acct/accounting.txt
	echo $2 >> $run_path/acct/accounting.txt
        echo "-----------" >> $run_path/acct/accounting.txt
	#echo "removed $2" | $mailxbin -s "vpn removed $2 - $weburl" ${mailcheck}
    }

function disablevpnuser
    {
        echo "- disabling user $2"
        mysql -Bsu $db_user -p$db_pass $db_name -e "update user set enabled=\"0\" where user_id=\"$2\"";
	#echo "removed $2" | $mailxbin -s "vpn removed $2 - $weburl" ${mailcheck}
    }

function enablevpnuser
    {
        echo "- disabling user $2"
        mysql -Bsu $db_user -p$db_pass $db_name -e "update user set enabled=\"1\" where user_id=\"$2\"";
	#echo "removed $2" | $mailxbin -s "vpn removed $2 - $weburl" ${mailcheck}
    }

function publicvpnset
    {
        echo "- public user settings for $2"
        url=`$md5sumbin $conf/$2.key|cut -f1 -d' '`
        echo $url
        mkdir /var/www/vpn/$url
        cp -vu $conf/$2.crt /var/www/vpn/$url/vpn.example.org.crt
        cp -vu $conf/$2.key /var/www/vpn/$url/vpn.example.org.key
	cp -vu $conf/ca.crt /var/www/vpn/$url/vpn.example.org-ca.crt
        cp -vu $run_path/vpn.example.org.ovpn /var/www/vpn/$url/
        cp -vu $run_path/vpn.example.org7.conf /var/www/vpn/$url/
        cp -vu $run_path/vpn.example.org.conf /var/www/vpn/$url/
        cp -vu $run_path/startvpn.bat /var/www/vpn/$url/
        cp -vu $run_path/startvpn7.bat /var/www/vpn/$url/
	cp -vu $run_path/README.* /var/www/vpn/$url/
	cd /var/www/vpn/$url
	zip -1 vpn.zip vpn.example.org.crt vpn.example.org-ca.crt vpn.example.org.key vpn.example.org.ovpn vpn.example.org.conf vpn.example.org7.conf startvpn.bat startvpn7.bat README.txt README.linux.txt README.windows.txt
	rm vpn.example.org.crt vpn.example.org-ca.crt vpn.example.org.key vpn.example.org.ovpn vpn.example.org.conf vpn.example.org7.conf startvpn.bat startvpn7.bat README.txt README.linux.txt README.windows.txt
	cd $run_path
	cp -vu $run_path/.htaccess /var/www/vpn/$url/
        echo `date` >> $run_path/acct/accounting.txt
        echo "Public :" >> $run_path/acct/accounting.txt
        echo $2 $weburl/$url >> $run_path/acct/accounting.txt
        echo "-----------" >> $run_path/acct/accounting.txt
	#echo "$weburl/$url" | $mailxbin -s "vpn settings" ${3}
	#echo "public $weburl/$url" | $mailxbin -s "vpn public - $weburl" ${mailcheck}
	# making sure that this URL will expire in $urltime
	echo "rm -rf /var/www/vpn/$url" > /tmp/vpnmgmt.rmurl
	at -m `date +%I%p` +$urltime hour < /tmp/vpnmgmt.rmurl
    }

# - run
cd $run_path

case $1 in
        add)
        addvpnuser default $2
        ;;
        remove)
        rmvpnuser default $2
        ;;
        public)
        publicvpnset default $2 $3
        ;;
	list)
	for i in `ls /etc/openvpn/*.csr`
	do user_id=`basename $i .csr`;
	    user_pass=`mysql -Bsu $db_user -p$db_pass $db_name -e "select user_pass from user where user_id=\"$user_id\""`
	    enabled=`mysql -Bsu $db_user -p$db_pass $db_name -e "select enabled from user where user_id=\"$user_id\""`
	    online=`mysql -Bsu $db_user -p$db_pass $db_name -e "select online from user where user_id=\"$user_id\""`
	    echo -en "user_id: $user_id\nuser_pass: $user_pass\nonline: $online\nenabled: $enabled\n-----------------\n"; done
	;;
        disable)
        disablevpnuser
        ;;
        enable)
        enablevpnuser
        ;;
        *)
	#echo "$1 - bad argument"
	#echo "$2 - bad argument"
	#echo "$3 - bad argument"
	echo "usage : vpnmgmt{.sh} {add/remove/enable/disable/public/list} {username,mailaddress}"
	echo "example : vpnmgmt add test - to add user test"
	echo "example : vpnmgmt public test test@email.com"
	echo "example : vpnmgmt list"
        exit 2
        ;;
esac

# - post_run cleanup
unset run_path
unset conf
unset md5sumbin
unset weburl
unset mailxbin
unset mailcheck
unset urltime
#unset mailto
exit 0

